#include "hdf5.h"
#include "stdlib.h"

int
main (int argc, char **argv)
{

int i;
int mpi_rank, mpi_size;
int rank = 2;
int *data;             /* pointer to data buffer to write */

hid_t file_id;         /* file identifier */
hid_t dset_id;         /* dataset identifiers */
hid_t filespace;       /* file dataspace identifiers */
hid_t filespace1;      /* file dataspace identifiers (hyperslab) */
hid_t memspace;        /* memory dataspace identifiers */
hid_t plist_id;        /* property list identifier */
hid_t plist2_id;        /* property list identifier */
hsize_t dimsf[rank];   /* global dimensions */
hsize_t count[rank];   /* local dimension (hyperslab) */
hsize_t offset[rank];  /* local offset (hyperslab) */
herr_t status;

char H5FILE_NAME[128]; /* filename */
char TASK_FILE[128]; /* task filename */

/* Initialize MPI */
MPI_Init(&argc, &argv);
MPI_Comm_size(MPI_COMM_WORLD, &mpi_size);
MPI_Comm_rank(MPI_COMM_WORLD, &mpi_rank);

/* setting filename */
sprintf(H5FILE_NAME,"RUN/%s.h5","my_fourth_parallel_file");

if (mpi_rank==0) printf("reading h5 file %s...", H5FILE_NAME);
if (mpi_rank==0) printf("...contigous hyperslab\n");

/* set global size */
dimsf[0] = 10;
dimsf[1] = 8;

/* Each task defines dataset in memory and writes it to the hyperslab in the file */
count[0] = dimsf[0]/mpi_size;
count[1] = dimsf[1];

/* Initialize data buffer  */
data = (int *) malloc(sizeof(int)*count[0]*count[1]);

//printf("I'm task %d of %d \n, %d, %d",mpi_rank,mpi_size,count[0],count[1]);
memspace = H5Screate_simple(rank, count, NULL);

/* set offset for each task */
offset[0] = mpi_rank * count[0];
offset[1] = 0;

/* Set up file access property list for MPI-IO access */
plist_id = H5Pcreate(H5P_FILE_ACCESS);
status = H5Pset_fapl_mpio(plist_id, MPI_COMM_WORLD, MPI_INFO_NULL);

file_id = H5Fopen(H5FILE_NAME, H5F_ACC_RDWR, plist_id);

/* open the dataset  */
//dset_id = H5Dopen(file_id, "dataset1");
// 1.8 API
 dset_id = H5Dopen(file_id, "dataset1",H5P_DEFAULT);

/* Select hyperslab in the file  */
filespace1 = H5Dget_space(dset_id);
status = H5Sselect_hyperslab(filespace1, H5S_SELECT_SET, offset, NULL, count, NULL);

/* Create property list for collective dataset read */ 
plist2_id = H5Pcreate(H5P_DATASET_XFER);
status = H5Pset_dxpl_mpio(plist2_id, H5FD_MPIO_COLLECTIVE);

/* read.... */
status = H5Dread(dset_id, H5T_NATIVE_INT, memspace, filespace1, plist2_id, data);
//status = H5Dread(dset_id, H5T_NATIVE_INT, memspace, filespace1, H5P_DEFAULT, data);

/* release property list identifier */
H5Pclose(plist_id);
H5Pclose(plist2_id);

/* close dataset */
H5Dclose(dset_id);

/* close */
H5Sclose(memspace);
H5Sclose(filespace1);

/* Close the file. */
H5Fclose(file_id);

/* write data buffer  */
sprintf(TASK_FILE,"RUN/%s.%d.dat","task",mpi_rank);
printf(" --> %s\n", TASK_FILE);
FILE *fid;
fid=fopen(TASK_FILE,"w");
if (fid!=NULL) {
   for (i=0; i < count[0]*count[1]; i++) {
       fprintf(fid,"i = %d, value --> %d \n", i, data[i]);
   }
}
fclose(fid);
MPI_Finalize();

}


